/*
 * flimsy_ui.cxx - Display and click-handling routines for flimsy
 *
 * $Id: flimsy_ui.cxx,v 1.34 2002/02/08 02:32:21 esm Exp $
 *
 * Copyright  2002 Eduardo Santiago Muoz
 *
 * This file is part of Flimsy.
 *
 * Flimsy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flimsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flimsy; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "defs.h"
#include "version.h"
#include "flimsy_ui.h"


/*
** Callbacks
*/
void flimsy_ui::done_cb(Fl_Widget *w, void *p)
{
  flimsy_ui *m;
  m = (flimsy_ui *)p;
  /* Anything to save? Do it now */
  exit(0);
}

/**************\
**  about_cb  **  Nearly useless... just invokes an About dialog
\**************/
void flimsy_ui::about_cb(Fl_Widget *w, void *p)
{
  About_Dialog( APPNAME, VERSION);
}


void flimsy_ui::show_by_movie(int movie_key)
{
  int n = 0;
  int n_wanted = 0;

  browser->clear();

  for (int m=0; movie_name[m]; m++) {
    char buf[1024];
    buf[0] = '@';
    buf[1] = 'b';
    strcpy(buf+2, movie_name[m]);
    browser->add(buf, 0);
    n++;			// Keep track of line number in browser

    if (m == movie_key) {
      n_wanted = n;
    }

    for (int i=0; showtimes[i]; i++) {
      if (showtimes[i][1] == m) {

	buf[0] = ' ';
	buf[1] = ' ';
	buf[2] = '\0';
	for (int j=0; j < showtimes[i][2]; j++) {
	  unsigned char mask = showtimes[i][3+2*j];
	  if (mask & todays_bit) {
	    strcat(buf, decode_t(showtimes[i][3+2*j+1]));
	    strcat(buf,",");
	  }
	}
	if (buf[2] != '\0') {
	  // Add a row for this theater.  FIXME: Although we include
	  // the "theater_info" callback, we don't display the '?'
	  // at right.  That's because I don't feel like declaring
	  // a new, different charbuf[].
	  browser->add(theater_info[showtimes[i][0]].name,
		       (void*)(CALLBACK_THEATER
			       | CALLBACK_THEATER_INFO
			       | showtimes[i][0]));
	  n++;

	  // Add a row for this set of times
	  buf[strlen(buf)-1] = '\0';
	  browser->add(buf, (void*)CALLBACK_TIMES);
	  n++;
	}
      }
    }
  }

  // Set the desired movie at the top of the list.
  browser->topline(n_wanted);
}



/*
**  show_by_theater  **  default way to show
*/
void flimsy_ui::show_by_theater(int theater_key)
{
  int n = 0;
  int n_wanted = 0;

  browser->clear();

  for (int t=0; theater_info[t].name; t++) {
    char buf[1024];

    buf[0] = '@';
    buf[1] = 'b';
    strcpy(buf+2, theater_info[t].name);
    strcat(buf,"\t@b@u?");
    browser->add(buf,(void*)(CALLBACK_THEATER_INFO | t));
    n++;

    if (t == theater_key) {
      n_wanted = n;
    }

    /* Movie names */
    for (int i=0; showtimes[i]; i++) {
      if (showtimes[i][0] == t) {
	buf[0] = ' ';

	/* Showtimes */
	buf[0] = ' ';
	buf[1] = ' ';
	buf[2] = '\0';
	for (int j=0; j < showtimes[i][2]; j++) {
	  unsigned char mask = showtimes[i][3+2*j];
	  if (mask & todays_bit) {
	    strcat(buf, decode_t(showtimes[i][3+2*j+1]));
	    strcat(buf, ",");
	  }
	}
	if (buf[2] != '\0') {
	  // Add a row for this movie name
	  browser->add(movie_name[showtimes[i][1]],
		       (void*)(CALLBACK_MOVIE | showtimes[i][1]));
	  n++;

	  // Add a row for this set of times (remember to trim the final comma)
	  buf[strlen(buf)-1] = '\0';
	  browser->add(buf, (void*)CALLBACK_TIMES);
	  n++;
	}
      }
    }
  }

  // Set the desired theater at the top of the list.
  browser->topline(n_wanted);
}


/***********************\
**  show_theater_info  **  Pop up a window with the theater's name,addr,phone
\***********************/
void flimsy_ui::show_theater_info( int key )
{
  fl_message("%s\n\n%s\n\n%s\n",
	     theater_info[key].name,
	     theater_info[key].address,
	     theater_info[key].phone);
}



void flimsy_ui::show_by_time( int t )
{
  // Obtain the encoded version of the current time.  With this, we'll set
  // the top of the browser to be the most recently past showtime.
  time_t    now = time(NULL);
  struct tm *tm = localtime(&now);
  unsigned char now_b = encode_t(tm->tm_hour*100 + tm->tm_min);

  // n = current line in browser, wanted = what we should set "top" to
  int n_wanted = 0;
  int n = 0;

  // Reset, and change the title
  browser->clear();

  for (int t=0; t <= 255; t++) {
    int did_t = 0;
    int last_theater = -1;

    for (int i=0; showtimes[i]; i++) {
      for (int j=0; j < showtimes[i][2]; j++) {
	if (showtimes[i][3+2*j+1] == t) {
	  unsigned char mask = showtimes[i][3+2*j];
	  if (mask & todays_bit) {
	    char buf[1024];

	    if (did_t++ == 0) {
	      buf[0] = '@';
	      buf[1] = 'b';
	      buf[2] = '\0';
	      strcat(buf, decode_t(t));
	      browser->add(buf, 0);

	      n++;
	      if (t < now_b) {
		n_wanted = n;
	      }
	    }

	    // Show the theater, if it isn't the same one as before
	    if (last_theater != showtimes[i][0]) {
	      strcpy(buf, theater_info[showtimes[i][0]].name);
	      strcat(buf, ":\t@b?");
	      browser->add(buf, (void*)(CALLBACK_THEATER
					| CALLBACK_THEATER_INFO
					| showtimes[i][0]));
	      n++;
	      last_theater = showtimes[i][0];
	    }

	    // Show the movie, and theater
	    buf[0] = ' ';
	    buf[1] = ' ';
	    buf[2] = '\0';
	    strcat(buf, movie_name[showtimes[i][1]]);
	    browser->add(buf, (void*)(CALLBACK_MOVIE | showtimes[i][1]));
	    n++;
	  }
	}
      }
    }
  }

  // Set the time closest to the present, at the top of the list.
  browser->topline(n_wanted);
}








void flimsy_ui::show(int argc, char **argv)
{
  /* FIXME: if called with args, maybe it's a date? */
  window->show();

  todays_bit = 1 << (7 - days_since_base());

  // No data for today?  Warn the user, but give him/her a chance to go on.
  if (!something_playing_on (todays_bit)) {
    int go_on = fl_ask("I have no listings for today.  My XML data file "
		       "is dated %s, and doesn't extend this far.  You "
		       "need to get new data using flimsy_get_listings.pl.\n"
		       "\n"
		       "There's little sense in doing so, but do you want "
		       "to continue?", friendly_date(0));
    if (go_on == 0) {
      exit (0);
    }
  }

  // Fill in the bottom-right menu, the one that gives a week's choices
  fill_in_date_menu();

  // Always start the same way, by listing theaters as major key.
  show_by_theater(0);
}



void browser_cb(Fl_Widget *o, void *p) {
  Fl_Browser *x = (Fl_Browser*)o;
  flimsy_ui *flimsy = (flimsy_ui*)p;
  int n = x->value();
  int data = (int)x->data(n);

  // Check if user hit the '?' at far right.  If so, pop up theater info
  if (data & CALLBACK_THEATER_INFO) {
    if (flimsy->column_widths[0] <= Fl::event_x()) {
      flimsy->show_theater_info( data & 0xFF );
      return;
    }
  }

  // Call the appropriate viewer
  if (data & CALLBACK_THEATER) {
    flimsy->show_by_theater( data & 0xFF );
  } else if (data & CALLBACK_MOVIE) {
    flimsy->show_by_movie( data & 0xFF );
  } else if (data & CALLBACK_TIMES) {
    flimsy->show_by_time( data & 0xFF );
  }
}




/*****************
**  Constructor  **
******************
**
** This is what creates the windowing interface.  I've decided to use
** the Agenda "flpda" routines, primarily because they hardcode the
** 160x220 so I don't have to (I hate hardcoding).  They also provide
** The little up-down pager buttons at bottom right.
*/
flimsy_ui::flimsy_ui()
{
  window = WidgetFactory::new_window("Flimsy");

  // FIXME: Make browser window shorter, but by how much?
  browser = WidgetFactory::new_paged_hold_browser(0, 0,
				window->w(),
				window->h() - WidgetFactory::buttonheight());
  browser->callback(browser_cb, this);

#if	!defined(BIG_FONT)
  browser->textsize(8);
#endif	// BIG_FONT

  // FIXME window->resizable(browser);

  toolbar = Widget_Factory::new_toolbar();
  WidgetFactory::new_button(_("Done"),    done_cb,    this);
  WidgetFactory::new_button(_("About"),   about_cb,   this);

  // One more button; this one is a menu one, and it takes up the rest
  // of the width of the taskbar (minus the pagekey buttons).  This will
  // have our list of weekdays.
  date_mb = WidgetFactory::new_menu_button(0,0);
  ((Fl_Widget*)date_mb)->size(
			      toolbar->w() - (2 * WidgetFactory::buttonwidth())
			      - WidgetFactory::pagekeywidth(),
			      date_mb->h());
  date_mb->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);

  Fl_Page_UpDown_Button* pb = WidgetFactory::new_pagekey();
  browser->page_updown_button(pb);

  column_widths = (int*)malloc(3 * sizeof(int));
  column_widths[0] = browser->w() - browser->textsize() - 1;
  column_widths[1] = browser->textsize();
  column_widths[2] =  0;
  browser->column_widths( column_widths );

  toolbar->end();
  window->add_dockable(toolbar, 1);
  window->end();

  for (int i=0; i < MAX_THEATERS; i++) {
    theater_info[i].name    = NULL;
    theater_info[i].address = NULL;
    theater_info[i].phone   = NULL;
  }
  for (int i=0; i < MAX_MOVIES; i++) {
    movie_name[i] = NULL;
  }
  for (int i=0; i < MAX_SHOWTIMES; i++) {
    showtimes[i] = NULL;
  }
  // Read in the data
  parse_data("flimsy.xml");
}


/*
** Destructor
*/
flimsy_ui::~flimsy_ui()
{
  delete(toolbar);
  delete(window);
}
