/*
 * flimsy_ui.h - class definition, plus a few helper functions & macros
 *
 * $Id: flimsy_ui.h,v 1.20 2002/02/07 03:20:55 esm Exp $
 *
 * Copyright  2002 Eduardo Santiago Muoz
 *
 * This file is part of Flimsy.
 *
 * Flimsy is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flimsy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flimsy; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef	FLIMSY_H
#define	FLIMSY_H 1

#include "defs.h"

class flimsy_ui
{
public:
  flimsy_ui();
  ~flimsy_ui();

  void show(int argc, char **argv);

  /* Helper functions */
  void parse_data(const char *filename);
  void show_by_theater(int key);
  void show_by_movie(int key);
  void show_by_time(int key);
  void show_theater_info(int key);
  void fill_in_date_menu( void );
  int  something_playing_on(unsigned char);

  /* Convert the current "todays_bit" to a string */
  char const *friendly_date(void);
  char const *friendly_date(int);
  int  days_since_base(void);

  /* Callbacks */
  static void done_cb (Fl_Widget*, void* data);
  static void about_cb(Fl_Widget*, void* data);
  static void date_change_cb(Fl_Widget*, void *data);

  /*
  ** Our data
  */
  struct theater_info {
    char *name;
    char *address;
    char *phone;
  } theater_info[MAX_THEATERS];
  char *movie_name[MAX_MOVIES];

  // FIXME: Document this...
  unsigned char *showtimes[MAX_SHOWTIMES];

  // year, month, date to use as reference base for the current data
  struct tm base_date;

  // Bitmask for today
  unsigned char todays_bit;

  // We use columns for indentation, and also as a special case for
  // displaying the "?" which pops up the theater info
  int *column_widths;

protected:
  Fl_App_Window          *window;	// The main window
  Fl_Paged_Hold_Browser  *browser;	// Big area with all the info
  Fl_Dockable_Window     *toolbar;	// "Done" button, menus, pager arrows
  Fl_Menu_Button         *date_mb;	// The pick-a-date menu button
};


unsigned char  encode_t( unsigned long );
char const    *decode_t( unsigned char );

/*
** Bit masks used to decide what to do in a callback
*/
#define CALLBACK_MOVIE		0x0100
#define CALLBACK_THEATER	0x0200
#define CALLBACK_TIMES		0x0400
#define	CALLBACK_THEATER_INFO	0x0800

#endif	// FLIMSY_H
